;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
;
; NAME: check_inside.pro
;
; PURPOSE: Find in which grid cell stands a given longitude/latitude position
;
; CATEGORY : Subroutine
;
; CALLING SEQUENCE : pro check_inside, xx,yy,zz, x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4,result
;
; INPUTS :
;         xx,yy,zz : cartesian coordinates of P the point that has to be located
;         in grid.
;         x1,y1,z1 : coordinates of A point of grid (See
;         documentation)
;         x2,y2,z2 : coordinates of B point of grid (See
;         documentation)
;         x3,y3,z3 : coordinates of C point of grid (See
;         documentation)
;         x4,y4,z4 : coordinates of D point of grid (See
;         documentation)
; OUTPUTS :
;         result : location of the P point in grid
; COMMON BLOCKS:
;         None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY: 08/2002 Robinson Hordoir
;
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------

pro check_inside, xx,yy,zz, x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4,result



ABx=x2-x1
ABy=y2-y1

BPx=xx-x2
BPy=yy-y2
BPz=zz-z2



AB_Vect_BP=ABx*BPy-BPx*ABy
BP=sqrt(BPx*BPx+BPy*BPy+BPz*BPz)


BCx=x3-x2
BCy=y3-y2

CPx=xx-x3
CPy=yy-y3
CPz=zz-z3


BC_Vect_CP=BCx*CPy-BCy*CPx
CP=sqrt(CPx*CPx+CPy*CPy+CPz*CPz)

CDx=x4-x3
CDy=y4-y3

DPx=xx-x4
DPy=yy-y4
DPz=zz-z4


CD_Vect_DP=CDx*DPy-CDy*DPx
DP=sqrt(DPx*DPx+DPy*DPy+DPz*DPz)

DAx=x1-x4
DAy=y1-y4

APx=xx-x1
APy=yy-y1
APz=zz-z1

DA_Vect_AP=DAx*APy-DAy*APx
AP=sqrt(APx*APx+APy*APy+APz*APz)

Dist=AP+BP+CP+DP



result= where( ( (AB_Vect_BP LE 0.) AND (BC_Vect_CP LE 0.) AND (CD_Vect_DP LE 0.) AND (DA_Vect_AP LE 0.) ) OR      $
               ( (AB_Vect_BP GE 0.) AND (BC_Vect_CP GE 0.) AND (CD_Vect_DP GE 0.) AND (DA_Vect_AP GE 0.) )  )                        



Dist_result=Dist(result)

min_dist=where(Dist_result EQ min(Dist_result))

result=result(min_dist)


end
