;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
;
; NAME: interp2.pro
;
; PURPOSE: Enables interpolation from a given ORCA grid to its
; closest geographical equivalent grid
;
; CATEGORY : Subroutine
;
; CALLING SEQUENCE : interp2
;
; INPUTS :
;         A full ORCA grid file + a 1D or 2D field on this grid 
; OUTPUTS :
;        - A grid file for the geographical equivalent to the given
;         ORCA grid   
;        - The interpolated field on this new grid + the weights and
;          pointers required to creat it.
;
; COMMON BLOCKS:
;
;         None
;
; SIDE EFFECTS:
;
; RESTRICTIONS: REQUIRES SAXO TOOL
;
; EXAMPLE:
;
; MODIFICATION HISTORY: 08/2002 Robinson Hordoir
;
;------------------------------------------------------------
pro interp2



@initorca
@naminterp2



;----------------------------------------------------------
; GEOGRID COMPUTATION (Grid & Mask)
;----------------------------------------------------------


if geogrid_mesh EQ 0 then begin

     print, 'Geogrid will be calculated, this may take a while, please wait...'

     make_geogrid

endif

;----------------------------------------------------------
; WEIGHT & POINTER FILE COMPUTATION
;----------------------------------------------------------

if weight_file EQ 0 then begin

     print, 'Weight file must calculated, this will take quite a while...'

     make_weight

endif

;----------------------------------------------------------
; Output fields calculation
;----------------------------------------------------------

     newinterp


end


